"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SFUConnection = void 0;
const common_1 = require("@skyway-sdk/common");
const core_1 = require("@skyway-sdk/core");
const receiver_1 = require("./receiver");
const sender_1 = require("./sender");
const log = new common_1.Logger('packages/sfu-client/src/connection/index.ts');
class SFUConnection {
    /**@internal */
    constructor(_api, channelId, localPerson, remoteMember, _transportRepository) {
        this._api = _api;
        this.channelId = channelId;
        this.localPerson = localPerson;
        this.remoteMember = remoteMember;
        this._transportRepository = _transportRepository;
        this.type = 'sfu';
        this.onDisconnect = new common_1.Event();
        this.onClose = new common_1.Event();
        this.closed = false;
        this._receivers = {};
        this._senders = {};
        this._consumeQueue = new common_1.PromiseQueue();
    }
    /**@internal */
    addSender(publication) {
        const sender = new sender_1.Sender(publication, this._api, this._transportRepository, this.localPerson.id, this.remoteMember, this.localPerson.iceManager);
        this._senders[publication.id] = sender;
        return sender;
    }
    /**@internal */
    removeSender(originPublicationId) {
        log.debug('removeSender', originPublicationId);
        const sender = this._senders[originPublicationId];
        if (sender) {
            sender.unproduce();
            delete this._senders[originPublicationId];
        }
    }
    /**@internal */
    startSubscribing(subscription) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._consumeQueue.push(() => this._startSubscribing(subscription));
        });
    }
    _startSubscribing(subscription) {
        return __awaiter(this, void 0, void 0, function* () {
            const receiver = new receiver_1.Receiver(subscription, this._api, this._transportRepository, this.localPerson.id, this.remoteMember, this.localPerson.iceManager);
            this._receivers[subscription.id] = receiver;
            const { codecs, track } = yield receiver.consume();
            const [selectedCodec] = codecs;
            // todo fix
            const stream = (0, core_1.createRemoteStream)('', '', track, selectedCodec);
            subscription.codec = {
                mimeType: selectedCodec.mimeType,
                parameters: selectedCodec.parameters,
            };
            subscription.stream = stream;
        });
    }
    /**@internal */
    stopSubscribing(subscription) {
        return __awaiter(this, void 0, void 0, function* () {
            const connection = this._receivers[subscription.id];
            if (!connection) {
                throw new common_1.SkyWayError({
                    type: 'notFound',
                    message: 'not exist connection',
                    payload: { subscription },
                    path: log.prefix,
                });
            }
            connection.unconsume();
            delete this._receivers[subscription.id];
        });
    }
    /**@internal */
    stopPublishing(publication) {
        return __awaiter(this, void 0, void 0, function* () {
            this.removeSender(publication.id);
        });
    }
    /**@internal */
    close() {
        if (this.closed) {
            return;
        }
        this.closed = true;
        Object.values(this._senders).forEach((s) => {
            s.close();
        });
        this._senders = {};
        this._receivers = {};
        this.onClose.emit();
    }
    /**@internal */
    getReceiver(subscriptionId) {
        return this._receivers[subscriptionId];
    }
    /**@internal */
    getSender(forwardingId) {
        return this._senders[forwardingId];
    }
    /**@internal */
    getRTCPeerConnection(subscriptionOrForwardingId) {
        var _a;
        const id = subscriptionOrForwardingId;
        const transceiver = (_a = this.getReceiver(id)) !== null && _a !== void 0 ? _a : this.getSender(id);
        return transceiver === null || transceiver === void 0 ? void 0 : transceiver.pc;
    }
}
exports.SFUConnection = SFUConnection;
//# sourceMappingURL=index.js.map