"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SfuBotMember = void 0;
const common_1 = require("@skyway-sdk/common");
const core_1 = require("@skyway-sdk/core");
const connection_1 = require("./connection");
const forwarding_1 = require("./forwarding");
const log = new common_1.Logger('packages/sfu-client/src/member.ts');
class SfuBotMember extends core_1.MemberImpl {
    /**@internal */
    constructor(args) {
        super(args);
        this.side = 'remote';
        this.subtype = SfuBotMember.subtype;
        this.type = 'bot';
        this._connections = {};
        this.onForwardingStarted = new common_1.Event();
        this.onForwardingStopped = new common_1.Event();
        this.onForwardingStateChanged = new common_1.Event();
        this._startForwardQueue = new common_1.PromiseQueue();
        this._forwardings = {};
        /**
         * @description {japanese} Forwardingを停止する
         */
        this.stopForwarding = (target) => new Promise((r, f) => {
            log.debug('[start] stopForwarding');
            if (this.status !== 'joined') {
                f(new common_1.SkyWayError({
                    type: 'forbidden',
                    message: 'sfuBot not in channel',
                    payload: { status: this.status },
                    path: log.prefix,
                }));
                return;
            }
            const forwardingId = typeof target === 'string' ? target : target.id;
            const forwarding = this._forwardings[forwardingId];
            if (!forwarding) {
                f(new common_1.SkyWayError({
                    type: 'notFound',
                    message: 'forwarding not found',
                    payload: { forwardingId },
                }));
                return;
            }
            delete this._forwardings[forwarding.id];
            const { promise, fulfilled } = this._api.stopForwarding({
                botId: this.id,
                forwardingId,
            });
            promise.catch(f);
            this.onForwardingStopped
                .watch((e) => e.forwarding.id === forwardingId, this._context.config.rtcApi.timeout)
                .catch((error) => {
                f(new common_1.SkyWayError({
                    type: 'timeout',
                    message: 'onForwardingStopped',
                    payload: { error, fulfilled },
                    path: log.prefix,
                }));
            })
                .then(() => {
                log.debug('[end] stopForwarding');
                r();
            });
        });
        this._api = args.api;
        this._context = args.context;
        this._transportRepository = args.transportRepository;
        this.onLeft.once(() => {
            log.debug('SfuBotMember left: ', { id: this.id });
            Object.values(this._connections).forEach((c) => {
                c.close();
            });
            this._connections = {};
        });
    }
    get forwardings() {
        return Object.values(this._forwardings);
    }
    /**@private */
    _getConnection(localPersonId) {
        return this._connections[localPersonId];
    }
    /**@private */
    _getOrCreateConnection(localPerson) {
        var _a;
        const connection = (_a = this._getConnection(localPerson.id)) !== null && _a !== void 0 ? _a : this._createConnection(this.channel, localPerson, this);
        return connection;
    }
    /**@private */
    _createConnection(channel, localPerson, endpointBot) {
        // todo fix
        if (!localPerson) {
            throw new common_1.SkyWayError({
                type: 'notFound',
                message: 'LocalPerson not found in channel instance.',
                path: log.prefix,
            });
        }
        const connection = new connection_1.SFUConnection(endpointBot._api, channel.id, localPerson, endpointBot, this._transportRepository);
        connection.onClose.once(() => {
            delete this._connections[localPerson.id];
        });
        this._connections[localPerson.id] = connection;
        return connection;
    }
    /**
     * @description {japanese} StreamのPublicationをForwardingする
     */
    startForwarding(publication, configure = { maxSubscribers: 4 }) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._startForwardQueue.push(() => this._startForwarding(publication, configure));
        });
    }
    _startForwarding(publication, configure = { maxSubscribers: 4 }) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.status !== 'joined') {
                throw new common_1.SkyWayError({
                    type: 'notFound',
                    message: 'sfuBot not in channel',
                    payload: { status: this.status },
                    path: log.prefix,
                });
            }
            if (!this.channel._getPublication(publication.id)) {
                throw new common_1.SkyWayError({
                    type: 'notFound',
                    message: 'publication not in channel',
                    payload: { status: this.status },
                    path: log.prefix,
                });
            }
            const relayed = publication;
            const localPerson = this.channel.localPerson;
            if (!localPerson) {
                throw new common_1.SkyWayError({
                    type: 'notFound',
                    message: 'localPerson not found in channel',
                });
            }
            if (localPerson.id !== relayed.publisher.id) {
                throw new common_1.SkyWayError({
                    type: 'invalidParameter',
                    message: 'publisherId not valid',
                });
            }
            log.debug('[start] SfuBotMember startForwarding', {
                publication: relayed.toJSON(),
                configure,
            });
            try {
                const connection = this._getOrCreateConnection(localPerson);
                const sender = connection.addSender(relayed);
                const forwardingId = yield sender.startForwarding(configure);
                let relayingPublication = this.channel._getPublication(forwardingId);
                if (!relayingPublication) {
                    relayingPublication = (yield this.channel.onStreamPublished
                        .watch((e) => e.publication.id === forwardingId, this._context.config.rtcApi.timeout)
                        .catch(() => {
                        throw new common_1.SkyWayError({
                            type: 'timeout',
                            message: 'SfuBotMember onStreamPublished',
                            payload: { forwardingId },
                            path: log.prefix,
                        });
                    })).publication;
                }
                const forwarding = new forwarding_1.Forwarding(configure, relayed, relayingPublication);
                this._forwardings[forwarding.id] = forwarding;
                this.listenStopForwardEvent(forwarding);
                this.onForwardingStarted.emit({ forwarding });
                this.onForwardingStateChanged.emit();
                log.debug('[end] SfuBotMember startForwarding', {
                    forwarding: forwarding.toJSON(),
                });
                return forwarding;
            }
            catch (error) {
                throw new common_1.SkyWayError({
                    type: 'internalError',
                    message: '[failed] SfuBotMember startForwarding',
                    payload: { error, publication },
                    path: log.prefix,
                });
            }
        });
    }
    listenStopForwardEvent(forwarding) {
        const { removeListener } = this.channel.onStreamUnpublished.add((e) => {
            if (e.publication.id === forwarding.id) {
                removeListener();
                forwarding.stop();
                const origin = forwarding.originPublication;
                const connection = this._getConnection(origin.publisher.id);
                if (connection) {
                    connection.removeSender(origin.id);
                }
                this.onForwardingStopped.emit({ forwarding });
                this.onForwardingStateChanged.emit();
            }
        });
    }
}
exports.SfuBotMember = SfuBotMember;
SfuBotMember.subtype = 'sfu';
/**
 * @description {japanese} MemberがSfuBotかどうか識別する
 */
SfuBotMember.IsSfuBotMember = (member) => {
    if (member.subtype === SfuBotMember.subtype) {
        return true;
    }
    return false;
};
//# sourceMappingURL=member.js.map